-- Install packer
local install_path = vim.fn.stdpath 'data' .. '/site/pack/packer/start/packer.nvim'
local is_bootstrap = false
if vim.fn.empty(vim.fn.glob(install_path)) > 0 then
  is_bootstrap = true
  vim.fn.system { 'git', 'clone', '--depth', '1', 'https://github.com/wbthomason/packer.nvim', install_path }
  vim.cmd [[packadd packer.nvim]]
end

-- IMPORTS
local helpers = require('helpers')

require('packer').startup(function(use)
  -- Package manager
  use 'wbthomason/packer.nvim'

  use { -- LSP Configuration & Plugins
    'neovim/nvim-lspconfig',
    requires = {
      -- Automatically install LSPs to stdpath for neovim
      'williamboman/mason.nvim',
      'williamboman/mason-lspconfig.nvim',

      'jose-elias-alvarez/null-ls.nvim',
      'jay-babu/mason-null-ls.nvim',

      -- Useful status updates for LSP
      { 'j-hui/fidget.nvim', tag = 'legacy' },

      -- Additional lua configuration, makes nvim stuff amazing
      'folke/neodev.nvim',
    },
  }

  use { -- Autocompletion
    'hrsh7th/nvim-cmp',
    requires = {
      'onsails/lspkind.nvim',
      'hrsh7th/cmp-nvim-lsp',
      'L3MON4D3/LuaSnip',
      'saadparwaiz1/cmp_luasnip',
      'hrsh7th/cmp-buffer', -- Buffer completion
      'hrsh7th/cmp-path', -- File path completion
      'hrsh7th/cmp-nvim-lua', -- Lua completion
    },
  }

  use { -- Highlight, edit, and navigate code
    'nvim-treesitter/nvim-treesitter',
    run = function()
      pcall(require('nvim-treesitter.install').update { with_sync = true })
    end,
  }

  use { -- Additional text objects via treesitter
    'nvim-treesitter/nvim-treesitter-textobjects',
    after = 'nvim-treesitter',
  }

  -- Git related plugins
  use 'tpope/vim-fugitive'
  use 'tpope/vim-rhubarb'
  use 'lewis6991/gitsigns.nvim'

  -- use 'navariau/onedark.nvim' -- Theme inspired by Atom
  use 'sainnhe/edge' -- Colorscheme
  use 'nvim-lualine/lualine.nvim' -- Fancier statusline
  use 'lukas-reineke/indent-blankline.nvim' -- Add indentation guides even on blank lines
  use 'numToStr/Comment.nvim' -- "gc" to comment visual regions/lines
  use 'tpope/vim-sleuth' -- Detect tabstop and shiftwidth automatically

  -- Fuzzy Finder (files, lsp, etc)
  use { 'nvim-telescope/telescope.nvim', branch = '0.1.x', requires = { 'nvim-lua/plenary.nvim' } }

  -- Fuzzy Finder Algorithm which requires local dependencies to be built. Only load if `make` is available
  use { 'nvim-telescope/telescope-fzf-native.nvim', run = 'make', cond = vim.fn.executable 'make' == 1 }
  use { 'nvim-telescope/telescope-ui-select.nvim' }
  use {
    "benfowler/telescope-luasnip.nvim",
    -- module = "telescope._extensions.luasnip", -- if you wish to lazy-load
  }

  use 'scrooloose/nerdtree'
  use 'jistr/vim-nerdtree-tabs'
  use 'Xuyuanp/nerdtree-git-plugin'
  use 'ryanoasis/vim-devicons'

  -- Easy window resize
  use 'roxma/vim-window-resize-easy'

  -- Session Management
  use 'tpope/vim-obsession'

  use 'maxbrunsfeld/vim-yankstack'
  use 'ntpeters/vim-better-whitespace'

  use "rafamadriz/friendly-snippets"

  use {
    "zbirenbaum/copilot.lua",
    cmd = "Copilot",
    event = "VimEnter",
    config = function()
      vim.schedule(function()
        require("copilot").setup({
          cmp = { enabled = true },
          suggestion = { enabled = false },
          panel = { enabled = false }
        })
      end)
    end,
  }

  use {
    "zbirenbaum/copilot-cmp",
    after = { "copilot.lua" },
    module = "copilot_cmp",
    config = function()
      require("copilot_cmp").setup()
    end
  }

  use {
    "folke/trouble.nvim",
    requires = "nvim-tree/nvim-web-devicons",
    config = function()
      require("trouble").setup {
        -- your configuration comes here
        -- or leave it empty to use the default settings
        -- refer to the configuration section below
      }
    end
  }

  use 'majutsushi/tagbar'

  -- Parens
  use {
    "windwp/nvim-autopairs",
    event = "InsertEnter",
    config = function() require("nvim-autopairs").setup {} end
  }

  use 'tpope/vim-surround'

  -- Todo comment highlight + diagnostics + navigation
  use {
    "folke/todo-comments.nvim",
    requires = "nvim-lua/plenary.nvim",
  }

  -- Smart Buffer management
  use {
    'akinsho/bufferline.nvim',
    tag = "v3.*",
    requires = 'nvim-tree/nvim-web-devicons',
  }
  use 'johann2357/nvim-smartbufs'

  -- LTeX local code actions
  use { "barreiroleo/ltex-extra.nvim" }

  -- DevOps Quatsch
  use { 'cespare/vim-toml', ft = 'toml' }
  use { 'mrk21/yaml-vim', ft = 'yaml' }
  use { 'hashivim/vim-terraform', ft = 'terraform' }
  use { 'pedrohdz/vim-yaml-folds', ft = 'yaml' }

  use {
    "folke/zen-mode.nvim",
    config = function()
      require("zen-mode").setup {
        -- your configuration comes here
        -- or leave it empty to use the default settings
        -- refer to the configuration section below
      }
    end
  }
  use {
    "SmiteshP/nvim-navic",
    requires = "neovim/nvim-lspconfig"
  }

  -- Add custom plugins to packer from ~/.config/nvim/lua/custom/plugins.lua
  local has_plugins, plugins = pcall(require, 'custom.plugins')
  if has_plugins then
    plugins(use)
  end

  if is_bootstrap then
    require('packer').sync()
  end
end)
-- When we are bootstrapping a configuration, it doesn't
-- make sense to execute the rest of the init.lua.
--
-- You'll need to restart nvim, and then it will work.
if is_bootstrap then
  print '=================================='
  print '    Plugins are being installed'
  print '    Wait until Packer completes,'
  print '       then restart nvim'
  print '=================================='
  return
end

-- Automatically source and re-compile packer whenever you save this init.lua
local packer_group = vim.api.nvim_create_augroup('Packer', { clear = true })
vim.api.nvim_create_autocmd('BufWritePost', {
  command = 'source <afile> | silent! LspStop | silent! LspStart | PackerCompile',
  group = packer_group,
  pattern = vim.fn.expand '$MYVIMRC',
})

-- [[ Setting options ]]
-- See `:help vim.o`

-- Set highlight on search
vim.o.hlsearch = false

vim.o.scrolloff = 7

-- Make line numbers default
vim.wo.number = true

-- Enable mouse mode
vim.o.mouse = 'nv'

-- Enable break indent
vim.o.breakindent = true

-- Save undo history
vim.o.undofile = true

vim.o.relativenumber = true
vim.o.encoding = 'utf-8'
vim.o.expandtab = true
vim.o.splitbelow = true
vim.o.splitright = true

-- Case insensitive searching UNLESS /C or capital in search
vim.o.ignorecase = true
vim.o.smartcase = true

-- Decrease update time
vim.o.updatetime = 250
vim.wo.signcolumn = 'yes'

-- Set colorscheme
vim.o.termguicolors = true
vim.cmd [[
  if exists("$LIGHTMODE")
    set bg=light
    let g:edge_style = 'light'
  else
    set bg=dark
    let g:edge_style = 'neon'
  endif
  let g:edge_better_performance = 1
  colorscheme edge
]]

-- Set completeopt to have a better completion experience
vim.o.completeopt = 'menuone,noselect'

-- [[ Basic Keymaps ]]
-- Set <space> as the leader key
-- See `:help mapleader`
--  NOTE: Must happen before plugins are required (otherwise wrong leader will be used)
vim.g.mapleader = ' '
vim.g.maplocalleader = ' '

-- Keymaps for better default experience
-- See `:help vim.keymap.set()`
vim.keymap.set({ 'n', 'v' }, '<Space>', '<Nop>', { silent = true })

-- Remap for dealing with word wrap
vim.keymap.set('n', 'k', "v:count == 0 ? 'gk' : 'k'", { expr = true, silent = true })
vim.keymap.set('n', 'j', "v:count == 0 ? 'gj' : 'j'", { expr = true, silent = true })

-- [[ Highlight on yank ]]
-- See `:help vim.highlight.on_yank()`
local highlight_group = vim.api.nvim_create_augroup('YankHighlight', { clear = true })
vim.api.nvim_create_autocmd('TextYankPost', {
  callback = function()
    vim.highlight.on_yank()
  end,
  group = highlight_group,
  pattern = '*',
})

require('nvim-navic').setup {
  icons = {
    File = ' ',
    Module = ' ',
    Namespace = ' ',
    Package = ' ',
    Class = ' ',
    Method = ' ',
    Property = ' ',
    Field = ' ',
    Constructor = ' ',
    Enum = ' ',
    Interface = ' ',
    Function = ' ',
    Variable = ' ',
    Constant = ' ',
    String = ' ',
    Number = ' ',
    Boolean = ' ',
    Array = ' ',
    Object = ' ',
    Key = ' ',
    Null = ' ',
    EnumMember = ' ',
    Struct = ' ',
    Event = ' ',
    Operator = ' ',
    TypeParameter = ' '
  },
  highlight = true,
}

-- Set lualine as statusline
-- See `:help lualine.txt`
require('lualine').setup {
  options = {
    icons_enabled = false,
    theme = 'onedark',
    component_separators = '|',
    section_separators = '',
  },
  sections = {
    lualine_c = {
      { require('nvim-navic').get_location, cond = require('nvim-navic').is_available }
    }
  }
}

require("bufferline").setup {}

-- Enable Comment.nvim
require('Comment').setup()

-- Enable `lukas-reineke/indent-blankline.nvim`
-- See `:help indent_blankline.txt`
require('indent_blankline').setup {
  char = '┊',
  show_trailing_blankline_indent = false,
}

-- Gitsigns
-- See `:help gitsigns.txt`
require('gitsigns').setup {
  signs = {
    add = { text = '+' },
    change = { text = '~' },
    delete = { text = '_' },
    topdelete = { text = '‾' },
    changedelete = { text = '~' },
  },
}

-- [[ Configure Telescope ]]
-- See `:help telescope` and `:help telescope.setup()`
local action_layout = require("telescope.actions.layout")
require('telescope').setup {
  defaults = {
    mappings = {
      n = {
        ["<M-p>"] = action_layout.toggle_preview
      },
      i = {
        ['<C-u>'] = false,
        ['<C-d>'] = false,
        ["<M-p>"] = action_layout.toggle_preview
      },
    },
  },
  extensions = {
    ["ui-select"] = {
      layout_strategy = "cursor",
      layout_config = {
        width = { 0.8, max = 120 },
        height = { 0.8, max = 20 },
      },
    },
  },
}

-- Enable telescope fzf native, if installed
pcall(require('telescope').load_extension, 'fzf')

require("telescope").load_extension("ui-select")
require("telescope").load_extension("luasnip")

-- See `:help telescope.builtin`
helpers.nmap('<leader>?', require('telescope.builtin').oldfiles, '[?] Find recently opened files')
helpers.nmap('<leader><space>', require('telescope.builtin').buffers, '[ ] Find existing buffers')
helpers.nmap('<leader>/', function()
  -- You can pass additional configuration to telescope to change theme, layout, etc.
  require('telescope.builtin').current_buffer_fuzzy_find(require('telescope.themes').get_dropdown {
    winblend = 10,
    previewer = false,
  })
end, { desc = '[/] Fuzzily search in current buffer]' })

helpers.nmap('<leader>sf', require('telescope.builtin').find_files, '[S]earch [F]iles')
helpers.nmap('<leader>sh', require('telescope.builtin').help_tags, '[S]earch [H]elp')
helpers.nmap('<leader>sw', require('telescope.builtin').grep_string, '[S]earch current [W]ord')
helpers.nmap('<leader>sg', require('telescope.builtin').live_grep, '[S]earch by [G]rep')
helpers.nmap('<leader>sd', require('telescope.builtin').diagnostics, '[S]earch [D]iagnostics')
helpers.nmap('<leader>sk', require('telescope.builtin').keymaps, '[S]earch [K]eymaps')
helpers.nmap('<leader>sb', require('telescope.builtin').buffers, '[S]earch [B]uffers')
helpers.nmap('<leader>st', "<ESC>:TodoTelescope keywords=TODO,FIXME<CR>", '[S]earch [T]odos')
helpers.nmap('<leader>ss', require('telescope').extensions.luasnip.luasnip, '[S]how [S]nippets')


-- [[ Configure Treesitter ]]
-- See `:help nvim-treesitter`
require('nvim-treesitter.configs').setup {
  -- Add languages to be installed here that you want installed for treesitter
  ensure_installed = { 'c', 'cpp', 'go', 'lua', 'python', 'rust', 'typescript', 'help', 'vim', 'latex', 'bibtex' },

  highlight = { enable = true },
  indent = { enable = true, disable = { 'python' } },
  incremental_selection = {
    enable = true,
    keymaps = {
      init_selection = '<c-space>',
      node_incremental = '<c-space>',
      scope_incremental = '<c-s>',
      node_decremental = '<c-backspace>',
    },
  },
  textobjects = {
    select = {
      enable = true,
      lookahead = true, -- Automatically jump forward to textobj, similar to targets.vim
      keymaps = {
        -- You can use the capture groups defined in textobjects.scm
        ['aa'] = '@parameter.outer',
        ['ia'] = '@parameter.inner',
        ['af'] = '@function.outer',
        ['if'] = '@function.inner',
        ['ac'] = '@class.outer',
        ['ic'] = '@class.inner',
      },
    },
    move = {
      enable = true,
      set_jumps = true, -- whether to set jumps in the jumplist
      goto_next_start = {
        [']m'] = '@function.outer',
        [']]'] = '@class.outer',
      },
      goto_next_end = {
        [']M'] = '@function.outer',
        [']['] = '@class.outer',
      },
      goto_previous_start = {
        ['[m'] = '@function.outer',
        ['[['] = '@class.outer',
      },
      goto_previous_end = {
        ['[M'] = '@function.outer',
        ['[]'] = '@class.outer',
      },
    },
    swap = {
      enable = true,
      swap_next = {
        ['<leader>a'] = '@parameter.inner',
      },
      swap_previous = {
        ['<leader>A'] = '@parameter.inner',
      },
    },
  },
}

require("todo-comments").setup {
  -- your configuration comes here
  -- or leave it empty to use the default settings
  -- refer to the configuration section below
}


-- Diagnostic keymaps
vim.keymap.set('n', '[d', vim.diagnostic.goto_prev)
vim.keymap.set('n', ']d', vim.diagnostic.goto_next)
vim.keymap.set('n', '<leader>e', vim.diagnostic.open_float)
vim.keymap.set('n', '<leader>q', vim.diagnostic.setloclist)

-- Old bindings TODO: Refactor

vim.keymap.set('n', '<F2>', '<ESC>:NERDTreeToggle<CR>')
vim.keymap.set('n', '<F4>', '<ESC>:TagbarToggle<CR>')

-- On Save run whitespace

vim.api.nvim_create_autocmd('BufEnter', {
  pattern = "*[^(.rmd|.snippets)]",
  command = "EnableStripWhitespaceOnSave"
})

vim.api.nvim_create_autocmd('BufEnter', {
  pattern = "*.snippets",
  command = "DisableStripWhitespaceOnSave"
})

-- Yankstack
vim.g.yankstack_map_keys = 0
vim.keymap.set('n', '<S-P>', '<Plug>yankstack_substitute_newer_paste')
vim.keymap.set('n', '<C-P>', '<Plug>yankstack_substitute_older_paste')


-- Window movement
vim.keymap.set('n', '<C-h>', '<C-w>h')
vim.keymap.set('n', '<C-j>', '<C-w>j')
vim.keymap.set('n', '<C-k>', '<C-w>k')
vim.keymap.set('n', '<C-l>', '<C-w>l')

vim.keymap.set("n", "]t", function()
  require("todo-comments").jump_next()
end, { desc = "Next todo comment" })

vim.keymap.set("n", "[t", function()
  require("todo-comments").jump_prev()
end, { desc = "Previous todo comment" })


helpers.nmap('<Leader>1', function() require("nvim-smartbufs").goto_buffer(1) end, "Go to buffer 1")
helpers.nmap('<Leader>2', function() require("nvim-smartbufs").goto_buffer(2) end, "Go to buffer 2")
helpers.nmap('<Leader>3', function() require("nvim-smartbufs").goto_buffer(3) end, "Go to buffer 3")
helpers.nmap('<Leader>4', function() require("nvim-smartbufs").goto_buffer(4) end, "Go to buffer 4")
helpers.nmap('<Leader>5', function() require("nvim-smartbufs").goto_buffer(5) end, "Go to buffer 5")
helpers.nmap('<Leader>6', function() require("nvim-smartbufs").goto_buffer(6) end, "Go to buffer 6")
helpers.nmap('<Leader>7', function() require("nvim-smartbufs").goto_buffer(7) end, "Go to buffer 7")
helpers.nmap('<Leader>8', function() require("nvim-smartbufs").goto_buffer(8) end, "Go to buffer 8")
helpers.nmap('<Leader>9', function() require("nvim-smartbufs").goto_buffer(9) end, "Go to buffer 9")

-- " Improved :bnext :bprev behavior (without considering terminal buffers)
helpers.nmap('<Right>', require("nvim-smartbufs").goto_next_buffer, 'Go to next buffer')
helpers.nmap('<Left>', require("nvim-smartbufs").goto_prev_buffer, 'Go to previous buffer')

-- " Delete current buffer and goes back to the previous one
helpers.nmap('<Leader>qb', require("nvim-smartbufs").close_current_buffer, 'Close current buffer')


-- LSP settings.
--  This function gets run when an LSP connects to a particular buffer.
local on_attach = function(client, bufnr)
  -- NOTE: Remember that lua is a real programming language, and as such it is possible
  -- to define small helper and utility functions so you don't have to repeat yourself

  if client.name == "ltex" then
    require("ltex_extra").setup {
      load_langs = { "de-DE", "en-US" }, -- table <string> : languages for witch dictionaries will be loaded
      init_check = true, -- boolean : whether to load dictionaries on startup
      path = "/home/rad4day/.config/nvim/ltex", -- string : path to store dictionaries. Relative path uses current working directory
      log_level = "none", -- string : "none", "trace", "debug", "info", "warn", "error", "fatal"
    }
  end

  -- helpers.diagnostic_on_cursor_hold(bufnr)

  local nmap = function(keys, func, desc)
    if desc then
      desc = 'LSP: ' .. desc
    end

    vim.keymap.set('n', keys, func, { buffer = bufnr, desc = desc })
  end

  nmap('<leader>rn', vim.lsp.buf.rename, '[R]e[n]ame')
  nmap('<leader>ca', vim.lsp.buf.code_action, '[C]ode [A]ction')

  nmap('gd', vim.lsp.buf.definition, '[G]oto [D]efinition')
  nmap('gr', require('telescope.builtin').lsp_references, '[G]oto [R]eferences')
  nmap('gI', vim.lsp.buf.implementation, '[G]oto [I]mplementation')
  nmap('<leader>D', vim.lsp.buf.type_definition, 'Type [D]efinition')
  nmap('<leader>ds', require('telescope.builtin').lsp_document_symbols, '[D]ocument [S]ymbols')
  nmap('<leader>ws', require('telescope.builtin').lsp_dynamic_workspace_symbols, '[W]orkspace [S]ymbols')

  -- See `:help K` for why this keymap
  nmap('K', vim.lsp.buf.hover, 'Hover Documentation')
  nmap('<C-k>', vim.lsp.buf.signature_help, 'Signature Documentation')

  -- Lesser used LSP functionality
  nmap('gD', vim.lsp.buf.declaration, '[G]oto [D]eclaration')
  nmap('<leader>wa', vim.lsp.buf.add_workspace_folder, '[W]orkspace [A]dd Folder')
  nmap('<leader>wr', vim.lsp.buf.remove_workspace_folder, '[W]orkspace [R]emove Folder')
  nmap('<leader>bf', vim.lsp.buf.format, '[B]uffer: [F]ormat')
  nmap('<leader>wl', function()
    print(vim.inspect(vim.lsp.buf.list_workspace_folders()))
  end, '[W]orkspace [L]ist Folders')

  -- Create a command `:Format` local to the LSP buffer
  vim.api.nvim_buf_create_user_command(bufnr, 'Format', function(_)
    vim.lsp.buf.format()
  end, { desc = 'Format current buffer with LSP' })

  -- Enable navic
  if client.server_capabilities.documentSymbolProvider then
    require('nvim-navic').attach(client, bufnr)
  end
end

-- Enable the following language servers
--  Feel free to add/remove any LSPs that you want here. They will automatically be installed.
--
--  Add any additional override configuration in the following tables. They will be passed to
--  the `settings` field of the server config. You must look up that documentation yourself.
local servers = {
  -- clangd = {},
  -- gopls = {},
  pyright = {},
  bashls = {},
  ansiblels = {},
  dockerls = {},
  marksman = {},
  rust_analyzer = {},
  yamlls = {},
  texlab = {}, -- latex
  terraformls = {},
  tflint = {},
  ltex = {
    ltex = {
      language = 'en-US',
      languageToolHttpServerUri = 'http://localhost:8081',
      checkFrequency = "save",
      additionalRules = {
        enablePickyRules = true,
        motherTongue = 'de-DE',
      },
      -- dictionary = {
      --   customEntries = {
      --     'foo',
      --     'bar',
      --   },
      -- },
    }, -- languagetool
  },
  -- tsserver = {},

  lua_ls = {
    Lua = {
      workspace = { checkThirdParty = false },
      telemetry = { enable = false },
    },
  },
}

local linters = {
  shellcheck = {},
  shfmt = {},
  yamllint = {},
  yamlfmt = {},
  black = {},
  hadolint = {}, -- Dockerfile
  gitlint = {},
  editorconfig_checker = {},
}
-- Setup neovim lua configuration
require('neodev').setup()
--
-- nvim-cmp supports additional completion capabilities, so broadcast that to servers
local capabilities = vim.lsp.protocol.make_client_capabilities()
capabilities = require('cmp_nvim_lsp').default_capabilities(capabilities)

-- Setup mason so it can manage external tooling
require('mason').setup()

-- Ensure the servers above are installed
local mason_lspconfig = require 'mason-lspconfig'
local mason_null_ls = require 'mason-null-ls'
local null_ls = require 'null-ls'


mason_lspconfig.setup {
  ensure_installed = vim.tbl_keys(servers),
}

mason_lspconfig.setup_handlers {
  function(server_name)
    require('lspconfig')[server_name].setup {
      capabilities = capabilities,
      on_attach = on_attach,
      settings = servers[server_name],
    }
  end,
}


mason_null_ls.setup {
  ensure_installed = vim.tbl_keys(linters),
  automatic_setup = true
}

null_ls.setup {
  sources = {
    null_ls.builtins.code_actions.gitsigns,
    null_ls.builtins.diagnostics.chktex,
  },
  on_attach = on_attach,
  root_dir = function(_) return nil end,
}

mason_null_ls.setup_handlers()


-- Turn on lsp status information
require('fidget').setup()

-- nvim-cmp setup
local cmp = require 'cmp'
local luasnip = require 'luasnip'
local lspkind = require 'lspkind'

vim.api.nvim_set_hl(0, "CmpItemKindCopilot", { fg = "#6CC644" })

cmp.setup {
  snippet = {
    expand = function(args)
      luasnip.lsp_expand(args.body)
    end,
  },
  window = {
    completion = cmp.config.window.bordered(),
    documentation = cmp.config.window.bordered(),
  },
  mapping = cmp.mapping.preset.insert {
    ['<C-d>'] = cmp.mapping.scroll_docs(-4),
    ['<C-f>'] = cmp.mapping.scroll_docs(4),
    ['<C-Space>'] = cmp.mapping.complete(),
    ['<CR>'] = cmp.mapping.confirm {
      behavior = cmp.ConfirmBehavior.Replace,
      select = false, -- false: only accept if explicitly selected. Otherwise <CR> after <dot> becomes a pita
    },
    ['<Tab>'] = cmp.mapping(function(fallback)
      if cmp.visible() then
        cmp.select_next_item()
      elseif luasnip.expand_or_jumpable() then
        luasnip.expand_or_jump()
      else
        fallback()
      end
    end, { 'i', 's' }),
    ['<S-Tab>'] = cmp.mapping(function(fallback)
      if cmp.visible() then
        cmp.select_prev_item()
      elseif luasnip.jumpable(-1) then
        luasnip.jump(-1)
      else
        fallback()
      end
    end, { 'i', 's' }),
  },
  sorting = {
    priority_weight = 2,
    comparators = {
      -- require("copilot_cmp.comparators").prioritize,
      -- require("copilot_cmp.comparators").score,
      --
      -- Below is the default comparitor list and order for nvim-cmp
      cmp.config.compare.offset,
      -- cmp.config.compare.scopes, --this is commented in nvim-cmp too
      cmp.config.compare.exact,
      cmp.config.compare.score,
      cmp.config.compare.recently_used,
      cmp.config.compare.locality,
      cmp.config.compare.kind,
      cmp.config.compare.sort_text,
      cmp.config.compare.length,
      cmp.config.compare.order,
    },
  },
  formatting = {
    format = lspkind.cmp_format({
      mode = 'symbol_text', -- show only symbol annotations
      maxwidth = 50, -- prevent the popup from showing more than provided characters (e.g 50 will not show more than 50 characters)
      ellipsis_char = '...', -- when popup menu exceed maxwidth, the truncated part would show ellipsis_char instead (must define maxwidth first)
      symbol_map = {
        Copilot = "",
      },

      -- The function below will be called before any actual modifications from lspkind
      -- so that you can provide more controls on popup customization. (See [#30](https://github.com/onsails/lspkind-nvim/pull/30))
      before = function(entry, vim_item)
        return vim_item
      end
    })
  },
  sources = {
    { name = 'copilot' },
    { name = 'nvim_lsp' },
    { name = 'luasnip' },
    { name = 'buffer' },
    { name = 'path' },
    { name = 'nvim_lua' },
  },
}

require("luasnip.loaders.from_vscode").lazy_load()

local hl = vim.api.nvim_set_hl
-- Navic

hl(0, "NavicIconsFile", { link = 'CmpItemKindFile' })
hl(0, "NavicIconsModule", { link = 'CmpItemKindModule' })
hl(0, "NavicIconsNamespace", { link = 'CmpItemKindModule' })
hl(0, "NavicIconsPackage", { link = 'CmpItemKindModule' })
hl(0, "NavicIconsClass", { link = 'CmpItemKindClass' })
hl(0, "NavicIconsMethod", { link = 'CmpItemKindMethod' })
hl(0, "NavicIconsProperty", { link = 'CmpItemKindProperty' })
hl(0, "NavicIconsField", { link = 'CmpItemKindField' })
hl(0, "NavicIconsConstructor", { link = 'CmpItemKindConstructor' })
hl(0, "NavicIconsEnum", { link = 'CmpItemKindEnum' })
hl(0, "NavicIconsInterface", { link = 'CmpItemKindInterface' })
hl(0, "NavicIconsFunction", { link = 'CmpItemKindFunction' })
hl(0, "NavicIconsVariable", { link = 'CmpItemKindVariable' })
hl(0, "NavicIconsConstant", { link = 'CmpItemKindConstant' })
hl(0, "NavicIconsString", { link = 'String' })
hl(0, "NavicIconsNumber", { link = 'Number' })
hl(0, "NavicIconsBoolean", { link = 'Boolean' })
hl(0, "NavicIconsArray", { link = 'CmpItemKindClass' })
hl(0, "NavicIconsObject", { link = 'CmpItemKindClass' })
hl(0, "NavicIconsKey", { link = 'CmpItemKindKeyword' })
hl(0, "NavicIconsKeyword", { link = 'CmpItemKindKeyword' })
hl(0, "NavicIconsNull", { link = '@none' })
hl(0, "NavicIconsEnumMember", { link = 'CmpItemKindEnumMember' })
hl(0, "NavicIconsStruct", { link = 'CmpItemKindStruct' })
hl(0, "NavicIconsEvent", { link = 'CmpItemKindEvent' })
hl(0, "NavicIconsOperator", { link = 'CmpItemKindOperator' })
hl(0, "NavicIconsTypeParameter", { link = 'CmpItemKindTypeParameter' })
hl(0, "NavicText", { link = '@keyword' })
hl(0, "NavicSeparator", { link = '@punctuation.delimiter' })

-- The line beneath this is called `modeline`. See `:help modeline`
-- vim: ts=2 sts=2 sw=2 et
